/********************************************************************************
 * (C) Copyright 2000-2005, by Shawn Qualia.
 *
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU Lesser General Public License as published by 
 * the Free Software Foundation; either version 2.1 of the License, or 
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public 
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License 
 * along with this library; if not, write to the Free Software Foundation, 
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc. 
 * in the United States and other countries.]
 ********************************************************************************/

package org.uguess.birt.report.engine.spreadsheet.model.impl;

import org.eclipse.birt.report.engine.content.IStyle;
import org.eclipse.birt.report.engine.css.dom.AbstractStyle;
import org.eclipse.birt.report.engine.css.engine.BIRTCSSEngine;
import org.eclipse.birt.report.engine.css.engine.StyleConstants;
import org.eclipse.birt.report.engine.css.engine.value.StringValue;
import org.w3c.dom.css.CSSValue;

/**
 * StyleImpl
 */
public class StyleImpl extends AbstractStyle
{

	private Object[] props = new Object[StyleConstants.NUMBER_OF_STYLE];
	private boolean empty = true;

	public static IStyle createEmpty( )
	{
		StyleImpl ds = new StyleImpl( );
		return ds;
	}

	public static IStyle create( IStyle style )
	{
		StyleImpl ds = new StyleImpl( );
		flattenParentStyles( ds, new IStyle[]{
			style
		} );
		return ds;
	}

	public static IStyle create( IStyle[] styles )
	{
		StyleImpl ds = new StyleImpl( );
		flattenParentStyles( ds, styles );
		return ds;
	}

	private static void flattenParentStyles( StyleImpl style, IStyle[] parents )
	{
		CSSValue value;
		if ( parents != null )
		{
			for ( int j = 0; j < StyleConstants.NUMBER_OF_STYLE; j++ )
			{
				value = style.getProperty( j );

				if ( isNonEffectiveValue( value ) )
				{
					for ( int i = 0; i < parents.length; i++ )
					{
						if ( parents[i] != null )
						{
							value = parents[i].getProperty( j );

							if ( !isNonEffectiveValue( value ) )
							{
								style.setProperty( j, value );
								break;
							}
						}
					}
				}
			}
		}
	}

	private static boolean isNonEffectiveValue( CSSValue value )
	{
		return value == null
				|| ( value instanceof StringValue && "transparent".equals( value.toString( ) ) ); //$NON-NLS-1$
	}

	private StyleImpl( )
	{
		super( new BIRTCSSEngine( ) );
	}

	public boolean isEmpty( )
	{
		return empty;
	}

	public int hashCode( )
	{
		int hash = 0;

		if ( !isEmpty( ) )
		{
			for ( int i = 0; i < StyleConstants.NUMBER_OF_STYLE; i++ )
			{
				CSSValue v1 = getProperty( i );

				if ( v1 != null )
				{
					hash += ( i ^ v1.hashCode( ) );
				}
			}
		}

		return hash;
	}

	public boolean equals( Object obj )
	{
		if ( obj == this )
		{
			return true;
		}

		if ( !( obj instanceof IStyle ) )
		{
			return false;
		}

		boolean aempty = ( (IStyle) obj ).isEmpty( );

		if ( empty && aempty )
		{
			return true;
		}

		if ( empty != aempty )
		{
			return false;
		}

		for ( int i = 0; i < StyleConstants.NUMBER_OF_STYLE; i++ )
		{
			CSSValue v1 = getProperty( i );
			CSSValue v2 = ( (IStyle) obj ).getProperty( i );

			if ( ( v1 == null && v2 != null ) || ( v2 == null && v1 != null ) )
			{
				return false;
			}

			if ( v1 != null && v2 != null && !v1.equals( v2 ) )
			{
				return false;
			}
		}

		return true;
	}

	public CSSValue getProperty( int index )
	{
		if ( index < 0 || index >= StyleConstants.NUMBER_OF_STYLE )
		{
			return null;
		}
		return (CSSValue) props[index];
	}

	public void setProperty( int index, CSSValue value )
	{
		if ( index < 0 || index >= StyleConstants.NUMBER_OF_STYLE )
		{
			return;
		}

		if ( empty )
		{
			empty = false;
		}

		props[index] = value;
	}
}
